@@ -1,5 +1,5 @@
 use bitflags::bitflags;
-use std::os::raw::c_int;
+use libc::c_int;
 
 #[cfg(any(target_os = "android", target_os = "linux"))]
 bitflags! {
@@ -55,11 +55,37 @@ bitflags! {
     }
 }
 
+#[cfg(not(target_os = "wasi"))]
+bitflags! {
+    /// `PROT_*` flags for use with [`mprotect`].
+    ///
+    /// [`mprotect`]: crate::io::mprotect
+    pub struct MprotectFlags: c_int {
+        /// `PROT_READ`
+        const READ = libc::PROT_READ;
+        /// `PROT_WRITE`
+        const WRITE = libc::PROT_WRITE;
+        /// `PROT_EXEC`
+        const EXEC = libc::PROT_EXEC;
+        /// `PROT_NONE`
+        const NONE = libc::PROT_NONE;
+        /// `PROT_GROWSUP`
+        #[cfg(any(target_os = "android", target_os = "linux"))]
+        const GROWSUP = libc::PROT_GROWSUP;
+        /// `PROT_GROWSDOWN`
+        #[cfg(any(target_os = "android", target_os = "linux"))]
+        const GROWSDOWN = libc::PROT_GROWSDOWN;
+    }
+}
+
 #[cfg(not(target_os = "wasi"))]
 bitflags! {
     /// `MAP_*` flags for use with [`mmap`].
     ///
+    /// For `MAP_ANONYMOUS` (aka `MAP_ANON`), see [`mmap_anonymous`].
+    ///
     /// [`mmap`]: crate::io::mmap
+    /// [`mmap_anonymous`]: crates::io::mmap_anonymous
     pub struct MapFlags: c_int {
         /// `MAP_SHARED`
         const SHARED = libc::MAP_SHARED;
@@ -78,8 +104,6 @@ bitflags! {
         const SHARED_VALIDATE = libc::MAP_SHARED_VALIDATE;
         /// `MAP_PRIVATE`
         const PRIVATE = libc::MAP_PRIVATE;
-        /// `MAP_ANONYMOUS`, aka `MAP_ANON`
-        const ANONYMOUS = libc::MAP_ANONYMOUS;
         /// `MAP_DENYWRITE`
         #[cfg(not(any(
             target_os = "ios",
@@ -102,6 +126,19 @@ bitflags! {
             target_os = "openbsd",
             target_os = "redox",
         )))]
+        const FIXED = libc::MAP_FIXED;
+        /// `MAP_FIXED_NOREPLACE`
+        #[cfg(not(any(
+            target_os = "android",
+            target_os = "emscripten",
+            target_os = "freebsd",
+            target_os = "fuchsia",
+            target_os = "ios",
+            target_os = "macos",
+            target_os = "netbsd",
+            target_os = "openbsd",
+            target_os = "redox",
+        )))]
         const FIXED_NOREPLACE = libc::MAP_FIXED_NOREPLACE;
         /// `MAP_GROWSDOWN`
         #[cfg(not(any(
@@ -199,6 +236,18 @@ bitflags! {
     }
 }
 
+#[cfg(any(target_os = "android", target_os = "linux"))]
+bitflags! {
+    /// `MLOCK_*` flags for use with [`mlock_with`].
+    ///
+    /// [`mlock_with`]: crate::io::mlock_with
+    pub struct MlockFlags: i32 {
+        // libc doesn't define `MLOCK_ONFAULT` yet.
+        // /// `MLOCK_ONFAULT`
+        // const ONFAULT = libc::MLOCK_ONFAULT;
+    }
+}
+
 #[cfg(not(any(target_os = "ios", target_os = "macos", target_os = "wasi")))]
 bitflags! {
     /// `O_*` constants for use with [`pipe_with`].
@@ -233,7 +282,7 @@ bitflags! {
     /// The `EFD_*` flags accepted by [`eventfd`].
     ///
     /// [`eventfd`]: crate::io::eventfd
-    pub struct EventfdFlags: std::os::raw::c_int {
+    pub struct EventfdFlags: libc::c_int {
         /// `EFD_CLOEXEC`
         const CLOEXEC = libc::EFD_CLOEXEC;
         /// `EFD_NONBLOCK`
@@ -245,9 +294,6 @@ bitflags! {
 
 /// `POSIX_MADV_*` constants for use with [`madvise`].
 ///
-/// Note that there is no `LinuxDontNeed` in the libc configuration because
-/// `libc` implementations don't provide a way to access it.
-///
 /// [`madvise`]: crate::io::madvise
 #[cfg(not(any(target_os = "redox", target_os = "wasi")))]
 #[derive(Debug, Copy, Clone, Eq, PartialEq)]
@@ -291,8 +337,18 @@ pub enum Advice {
 
     /// `POSIX_MADV_DONTNEED`
     #[cfg(target_os = "android")]
-    DontNeed = libc::MADV_DONTNEED,
+    DontNeed = i32::MAX - 1,
 
+    /// `MADV_DONTNEED`
+    // `MADV_DONTNEED` has the same value as `POSIX_MADV_DONTNEED`. We don't
+    // have a separate `posix_madvise` from `madvise`, so we expose a special
+    // value which we special-case.
+    #[cfg(target_os = "linux")]
+    LinuxDontNeed = i32::MAX,
+
+    /// `MADV_DONTNEED`
+    #[cfg(target_os = "android")]
+    LinuxDontNeed = libc::MADV_DONTNEED,
     /// `MADV_FREE`
     #[cfg(any(target_os = "android", target_os = "linux"))]
     LinuxFree = libc::MADV_FREE,
@@ -309,7 +365,11 @@ pub enum Advice {
     #[cfg(any(target_os = "android", target_os = "linux"))]
     LinuxHwPoison = libc::MADV_HWPOISON,
     /// `MADV_SOFT_OFFLINE`
-    #[cfg(any(target_os = "android", target_os = "linux"))]
+    // TODO: Enable riscv once <https://github.com/rust-lang/libc/pull/2391> lands.
+    #[cfg(all(
+        not(target_arch = "riscv64"),
+        any(target_os = "android", target_os = "linux")
+    ))]
     LinuxSoftOffline = libc::MADV_SOFT_OFFLINE,
     /// `MADV_MERGEABLE`
     #[cfg(any(target_os = "android", target_os = "linux"))]
