@@ -9,7 +9,7 @@
 )))]
 use super::conv::ret_u32;
 use super::conv::{
-    borrowed_fd, ret, ret_c_int, ret_discarded_fd, ret_off_t, ret_owned_fd, ret_ssize_t,
+    borrowed_fd, no_fd, ret, ret_c_int, ret_discarded_fd, ret_off_t, ret_owned_fd, ret_ssize_t,
 };
 #[cfg(any(target_os = "android", target_os = "linux"))]
 use super::conv::{syscall_ret, syscall_ret_owned_fd, syscall_ret_ssize_t};
@@ -42,6 +42,8 @@ use super::fs::{RenameFlags, ResolveFlags};
 use super::fs::{Statx, StatxFlags};
 #[cfg(not(any(target_os = "redox", target_os = "wasi")))]
 use super::io::Advice as IoAdvice;
+#[cfg(any(target_os = "android", target_os = "linux"))]
+use super::io::MlockFlags;
 #[cfg(not(any(target_os = "ios", target_os = "macos", target_os = "wasi")))]
 use super::io::PipeFlags;
 use super::io::PollFd;
@@ -80,13 +82,18 @@ use super::offset::libc_posix_fallocate;
 use super::offset::{libc_fstat, libc_fstatat, libc_lseek, libc_off_t, libc_pread, libc_pwrite};
 #[cfg(all(target_os = "linux", target_env = "gnu"))]
 use super::offset::{libc_preadv2, libc_pwritev2};
+#[cfg(not(target_os = "wasi"))]
+use super::process::RawUname;
 #[cfg(target_os = "linux")]
 use super::rand::GetRandomFlags;
 use super::time::Timespec;
 use crate::as_ptr;
 use crate::io::{self, OwnedFd, RawFd};
+#[cfg(not(target_os = "wasi"))]
+use crate::process::{Gid, Pid, Uid};
 use errno::errno;
 use io_lifetimes::{AsFd, BorrowedFd};
+use libc::{c_int, c_void};
 use std::cmp::min;
 use std::convert::TryInto;
 use std::ffi::CStr;
@@ -96,7 +103,6 @@ use std::io::{IoSlice, IoSliceMut, SeekFrom};
 #[cfg(target_os = "linux")]
 use std::mem::transmute;
 use std::mem::{size_of, MaybeUninit};
-use std::os::raw::{c_int, c_void};
 #[cfg(all(unix, not(target_os = "fuchsia")))]
 use std::os::unix::ffi::OsStringExt;
 #[cfg(target_os = "wasi")]
@@ -125,7 +131,7 @@ use {
 };
 #[cfg(not(target_os = "wasi"))]
 use {
-    super::io::{DupFlags, MapFlags, ProtFlags, Termios, Winsize},
+    super::io::{DupFlags, MapFlags, MprotectFlags, ProtFlags, Termios, Winsize},
     super::time::{ClockId, DynamicClockId},
 };
 
@@ -340,6 +346,12 @@ fn max_iov() -> usize {
     16 // The minimum value required by POSIX.
 }
 
+#[cfg(any(target_os = "android", target_os = "linux"))]
+#[inline]
+pub(crate) fn exit_group(code: c_int) -> ! {
+    unsafe { libc::_exit(code) }
+}
+
 pub(crate) unsafe fn close(raw_fd: RawFd) {
     let _ = libc::close(raw_fd as c_int);
 }
@@ -693,6 +705,14 @@ pub(crate) fn fadvise(
 
 #[cfg(not(any(target_os = "redox", target_os = "wasi")))]
 pub(crate) fn madvise(addr: *mut c_void, len: usize, advice: IoAdvice) -> io::Result<()> {
+    // On Linux platforms, `MADV_DONTNEED` has the same value as
+    // `POSIX_MADV_DONTNEED` but different behavior. We remap it to a different
+    // value, and check for it here.
+    #[cfg(target_os = "linux")]
+    if let IoAdvice::LinuxDontNeed = advice {
+        return unsafe { ret(libc::madvise(addr, len, libc::MADV_DONTNEED)) };
+    }
+
     #[cfg(not(target_os = "android"))]
     {
         let err = unsafe { libc::posix_madvise(addr, len, advice as libc::c_int) };
@@ -707,7 +727,13 @@ pub(crate) fn madvise(addr: *mut c_void, len: usize, advice: IoAdvice) -> io::Re
 
     #[cfg(target_os = "android")]
     {
-        unsafe { ret(libc::madvise(addr, len, advice as libc::c_int)) }
+        if let IoAdvice::DontNeed = advice {
+            // Do nothing. Linux's `MADV_DONTNEED` isn't the same as
+            // `POSIX_MADV_DONTNEED`, so just discard `MADV_DONTNEED`.
+            Ok(())
+        } else {
+            unsafe { ret(libc::madvise(addr, len, advice as libc::c_int)) }
+        }
     }
 }
 
@@ -1260,11 +1286,81 @@ pub(crate) unsafe fn mmap(
     }
 }
 
+/// # Safety
+///
+/// `mmap` is primarily unsafe due to the `addr` parameter, as anything working
+/// with memory pointed to by raw pointers is unsafe.
+#[cfg(not(target_os = "wasi"))]
+pub(crate) unsafe fn mmap_anonymous(
+    ptr: *mut c_void,
+    len: usize,
+    prot: ProtFlags,
+    flags: MapFlags,
+) -> io::Result<*mut c_void> {
+    let res = libc_mmap(
+        ptr,
+        len,
+        prot.bits(),
+        flags.bits() | libc::MAP_ANONYMOUS,
+        no_fd(),
+        0,
+    );
+    if res == libc::MAP_FAILED {
+        Err(io::Error::last_os_error())
+    } else {
+        Ok(res)
+    }
+}
+
+#[cfg(not(target_os = "wasi"))]
+pub(crate) unsafe fn mprotect(
+    ptr: *mut c_void,
+    len: usize,
+    flags: MprotectFlags,
+) -> io::Result<()> {
+    ret(libc::mprotect(ptr, len, flags.bits()))
+}
+
 #[cfg(not(target_os = "wasi"))]
 pub(crate) unsafe fn munmap(ptr: *mut c_void, len: usize) -> io::Result<()> {
     ret(libc::munmap(ptr, len))
 }
 
+/// # Safety
+///
+/// `mlock` operates on raw pointers and may round out to the nearest page
+/// boundaries.
+#[cfg(not(target_os = "wasi"))]
+#[inline]
+pub(crate) unsafe fn mlock(addr: *mut c_void, length: usize) -> io::Result<()> {
+    ret(libc::mlock(addr, length))
+}
+
+/// # Safety
+///
+/// `mlock_with` operates on raw pointers and may round out to the nearest page
+/// boundaries.
+#[cfg(any(target_os = "android", target_os = "linux"))]
+#[inline]
+pub(crate) unsafe fn mlock_with(
+    addr: *mut c_void,
+    length: usize,
+    flags: MlockFlags,
+) -> io::Result<()> {
+    assert_eq!(flags.bits(), 0, "libc doesn't define `MLOCK_*` yet");
+    ret(libc::mlock(addr, length))
+}
+
+/// # Safety
+///
+/// `munlock` operates on raw pointers and may round out to the nearest page
+/// boundaries.
+#[cfg(not(target_os = "wasi"))]
+#[inline]
+pub(crate) unsafe fn munlock(addr: *mut c_void, length: usize) -> io::Result<()> {
+    ret(libc::munlock(addr, length))
+}
+
 #[cfg(not(target_os = "wasi"))]
 pub(crate) fn pipe() -> io::Result<(OwnedFd, OwnedFd)> {
     unsafe {
@@ -1934,53 +2030,71 @@ pub(crate) fn fcntl_rdadvise(fd: BorrowedFd<'_>, offset: u64, len: u64) -> io::R
 #[cfg(not(target_os = "wasi"))]
 #[inline]
 #[must_use]
-pub(crate) fn getuid() -> u32 {
-    unsafe { libc::getuid() }
+pub(crate) fn getuid() -> Uid {
+    unsafe {
+        let uid = libc::getuid();
+        Uid::from_raw(uid)
+    }
 }
 
 #[cfg(not(target_os = "wasi"))]
 #[inline]
 #[must_use]
-pub(crate) fn geteuid() -> u32 {
-    unsafe { libc::geteuid() }
+pub(crate) fn geteuid() -> Uid {
+    unsafe {
+        let uid = libc::geteuid();
+        Uid::from_raw(uid)
+    }
 }
 
 #[cfg(not(target_os = "wasi"))]
 #[inline]
 #[must_use]
-pub(crate) fn getgid() -> u32 {
-    unsafe { libc::getgid() }
+pub(crate) fn getgid() -> Gid {
+    unsafe {
+        let gid = libc::getgid();
+        Gid::from_raw(gid)
+    }
 }
 
 #[cfg(not(target_os = "wasi"))]
 #[inline]
 #[must_use]
-pub(crate) fn getegid() -> u32 {
-    unsafe { libc::getegid() }
+pub(crate) fn getegid() -> Gid {
+    unsafe {
+        let gid = libc::getegid();
+        Gid::from_raw(gid)
+    }
 }
 
 #[cfg(not(target_os = "wasi"))]
 #[inline]
 #[must_use]
-pub(crate) fn getpid() -> u32 {
-    let pid: i32 = unsafe { libc::getpid() };
-    pid as u32
+pub(crate) fn getpid() -> Pid {
+    unsafe {
+        let pid = libc::getpid();
+        Pid::from_raw(pid)
+    }
 }
 
 #[cfg(not(target_os = "wasi"))]
 #[inline]
 #[must_use]
-pub(crate) fn getppid() -> u32 {
-    let pid: i32 = unsafe { libc::getppid() };
-    pid as u32
+pub(crate) fn getppid() -> Pid {
+    unsafe {
+        let pid: i32 = libc::getppid();
+        Pid::from_raw(pid)
+    }
 }
 
 #[cfg(any(target_os = "android", target_os = "linux"))]
 #[inline]
 #[must_use]
-pub(crate) fn gettid() -> u32 {
-    let tid: i32 = unsafe { libc::gettid() };
-    tid as u32
+pub(crate) fn gettid() -> Pid {
+    unsafe {
+        let tid: i32 = libc::gettid();
+        Pid::from_raw(tid)
+    }
 }
 
 #[inline]
@@ -1989,3 +2103,97 @@ pub(crate) fn sched_yield() {
         let _ = libc::sched_yield();
     }
 }
+
+#[cfg(not(target_os = "wasi"))]
+#[inline]
+pub(crate) fn uname() -> RawUname {
+    let mut uname = MaybeUninit::<RawUname>::uninit();
+    unsafe {
+        ret(libc::uname(uname.as_mut_ptr())).unwrap();
+        uname.assume_init()
+    }
+}
+
+#[cfg(not(any(target_os = "fuchsia", target_os = "wasi")))]
+#[inline]
+pub(crate) fn nice(inc: i32) -> io::Result<i32> {
+    errno::set_errno(errno::Errno(0));
+    let r = unsafe { libc::nice(inc) };
+    if errno::errno().0 != 0 {
+        ret_c_int(r)
+    } else {
+        Ok(r)
+    }
+}
+
+#[cfg(not(any(target_os = "fuchsia", target_os = "redox", target_os = "wasi")))]
+#[inline]
+pub(crate) fn getpriority_user(uid: Uid) -> io::Result<i32> {
+    errno::set_errno(errno::Errno(0));
+    let r = unsafe { libc::getpriority(libc::PRIO_USER, uid.as_raw() as _) };
+    if errno::errno().0 != 0 {
+        ret_c_int(r)
+    } else {
+        Ok(r)
+    }
+}
+
+#[cfg(not(any(target_os = "fuchsia", target_os = "redox", target_os = "wasi")))]
+#[inline]
+pub(crate) fn getpriority_pgrp(pgid: Pid) -> io::Result<i32> {
+    errno::set_errno(errno::Errno(0));
+    let r = unsafe { libc::getpriority(libc::PRIO_PGRP, pgid.as_raw() as _) };
+    if errno::errno().0 != 0 {
+        ret_c_int(r)
+    } else {
+        Ok(r)
+    }
+}
+
+#[cfg(not(any(target_os = "fuchsia", target_os = "redox", target_os = "wasi")))]
+#[inline]
+pub(crate) fn getpriority_process(pid: Pid) -> io::Result<i32> {
+    errno::set_errno(errno::Errno(0));
+    let r = unsafe { libc::getpriority(libc::PRIO_PROCESS, pid.as_raw() as _) };
+    if errno::errno().0 != 0 {
+        ret_c_int(r)
+    } else {
+        Ok(r)
+    }
+}
+
+#[cfg(not(any(target_os = "fuchsia", target_os = "redox", target_os = "wasi")))]
+#[inline]
+pub(crate) fn setpriority_user(uid: Uid, priority: i32) -> io::Result<()> {
+    unsafe {
+        ret(libc::setpriority(
+            libc::PRIO_USER,
+            uid.as_raw() as _,
+            priority,
+        ))
+    }
+}
+
+#[cfg(not(any(target_os = "fuchsia", target_os = "redox", target_os = "wasi")))]
+#[inline]
+pub(crate) fn setpriority_pgrp(pgid: Pid, priority: i32) -> io::Result<()> {
+    unsafe {
+        ret(libc::setpriority(
+            libc::PRIO_PGRP,
+            pgid.as_raw() as _,
+            priority,
+        ))
+    }
+}
+
+#[cfg(not(any(target_os = "fuchsia", target_os = "redox", target_os = "wasi")))]
+#[inline]
+pub(crate) fn setpriority_process(pid: Pid, priority: i32) -> io::Result<()> {
+    unsafe {
+        ret(libc::setpriority(
+            libc::PRIO_PROCESS,
+            pid.as_raw() as _,
+            priority,
+        ))
+    }
+}
