@@ -325,7 +325,7 @@ extern "C" {
     ) -> RetReg<R0>;
 }
 
-fn init() {
+fn minimal_init() {
     // Safety: Store default function addresses in static storage so that if we
     // end up making any system calls while we read the vDSO, they'll work.
     // If the memory happens to already be initialized, this is redundant, but
@@ -344,6 +344,10 @@ fn init() {
             .compare_exchange(0, rsix_int_0x80 as SyscallType as usize, Relaxed, Relaxed)
             .ok();
     }
+}
+
+fn init() {
+    minimal_init();
 
     if let Some(vdso) = vdso::Vdso::new() {
         #[cfg(target_arch = "x86_64")]
