@@ -13,7 +13,7 @@ pub(crate) use std::os::wasi::io::{AsRawFd, FromRawFd, IntoRawFd, RawFd};
 
 mod close;
 mod error;
-#[cfg(any(target_os = "android", target_os = "linux"))]
+#[cfg(any(linux_raw, all(libc, any(target_os = "android", target_os = "linux"))))]
 mod eventfd;
 mod fd;
 mod ioctl;
@@ -29,12 +29,12 @@ mod poll;
 mod procfs;
 mod read_write;
 mod stdio;
-#[cfg(any(target_os = "android", target_os = "linux"))]
+#[cfg(any(linux_raw, all(libc, any(target_os = "android", target_os = "linux"))))]
 mod userfaultfd;
 
 pub use close::close;
 pub use error::{Error, Result};
-#[cfg(any(target_os = "android", target_os = "linux"))]
+#[cfg(any(linux_raw, all(libc, any(target_os = "android", target_os = "linux"))))]
 pub use eventfd::{eventfd, EventfdFlags};
 #[cfg(not(target_os = "redox"))]
 pub use fd::ioctl_fionread;
@@ -45,7 +45,7 @@ pub use fd::isatty;
 pub use fd::ttyname;
 #[cfg(not(target_os = "wasi"))]
 pub use fd::{dup, dup2, dup2_with, DupFlags};
-#[cfg(any(target_os = "android", target_os = "linux"))]
+#[cfg(any(linux_raw, all(libc, any(target_os = "android", target_os = "linux"))))]
 pub use imp::io::epoll;
 #[cfg(any(target_os = "ios", target_os = "macos"))]
 pub use ioctl::ioctl_fioclex;
@@ -60,15 +60,19 @@ pub use ioctl::{ioctl_tiocexcl, ioctl_tiocnxcl};
 #[cfg(not(any(target_os = "redox", target_os = "wasi")))]
 pub use madvise::{madvise, Advice};
 #[cfg(not(target_os = "wasi"))]
-pub use mmap::{mmap, munmap, MapFlags, ProtFlags};
+pub use mmap::{
+    mlock, mmap, mmap_anonymous, mprotect, munlock, munmap, MapFlags, MprotectFlags, ProtFlags,
+};
+#[cfg(any(linux_raw, all(libc, any(target_os = "android", target_os = "linux"))))]
+pub use mmap::{mlock_with, MlockFlags};
 pub use owned_fd::OwnedFd;
 #[cfg(not(target_os = "wasi"))]
 pub use pipe::pipe;
 #[cfg(not(any(target_os = "ios", target_os = "macos", target_os = "wasi")))]
 pub use pipe::{pipe_with, PipeFlags};
-pub use poll::{PollFd, PollFdVec, PollFlags};
+pub use poll::{poll, PollFd, PollFlags};
 #[cfg(any(target_os = "android", target_os = "linux"))]
-pub use procfs::{proc, proc_self, proc_self_fd};
+pub use procfs::proc_self_fd;
 pub use read_write::{pread, pwrite, read, readv, write, writev};
 #[cfg(not(target_os = "redox"))]
 pub use read_write::{preadv, pwritev};
@@ -78,9 +82,6 @@ pub use stdio::{stderr, stdin, stdout, take_stderr, take_stdin, take_stdout};
 #[cfg(any(linux_raw, all(libc, any(target_os = "android", target_os = "linux"))))]
 pub use userfaultfd::{userfaultfd, UserfaultfdFlags};
 
-#[cfg(all(linux_raw, any(target_os = "android", target_os = "linux")))]
-pub(crate) use procfs::proc_self_auxv;
-
 #[cfg(any(linux_raw, not(target_os = "wasi")))]
 pub use imp::io::Termios;
 
